#!-PERL-
# mif2fm.pl
# Copyright : Norwegian Telecom Research, 1993
# Programming : Jon Stephenson von Tetzchner
# This file can be changed and modified as long as this
# header is kept intact. This code can not be included
# in a commercial product.

# This script is part of the fm2html package.

$| = 1;

$, = "\n";		# set output field separator
$\ = "\n";		# set output record separator

$FM_MAGIC	= "<MakerFile";
$FM_BOOK_MAGIC	= "<Bookfile";
$MIF_MAGIC	= "<MIFFile";
$MIF_BOOK_MAGIC	= "<Book ";

$TMPFNAME	= "mif2fm.fmbatch";

local( $tmpdir, $tmpfile, $infile, $outfile, $mif, $doc, @inputs );

$tmpdir = $ENV{'TMPFDIR'};
if ( ! $tmpdir ) { $tmpdir = "-TMPDIR-"; }
if ( ! $tmpdir ) { $tmpdir = `pwd`; }
$tmpdir =~ s#/$##;
$tmpfile = "$tmpdir/$TMPFNAME.$$";

while ( $ARGV[0] =~ m|^-| )
{
  $_ = shift;
  last if m|^--|;
  
  die "Invalid flag: \"$_\"";
}

if ( $#ARGV != 0 )
{
  print "\n\tUsage: fm2mif mif-file";
  print "";
  print "\tThe parameter specifies the name of a FrameMaker MIF file to";
  print "\ttranslate to a binary FrameMaker document file.  If a FrameMaker";
  print "\tbook file is specified then all files in the book will be";
  print "\ttranslated, as will the book itself.";
  print "";
  print "\tThe name of each output file is formed by appending \".doc\"";
  print "\tto the name of the input file.  If the name of an input file";
  print "\tends in a \".mif\" extension, that extension will be removed";
  print "\tprior to \".doc\" being appended.";
  print "";
  exit( 1 );
}

$infile = $ARGV[0];

open( TMPF, ">$tmpfile" )
  || die "fm2mif: ERROR: Could not create temporary file $tmpfile... exiting";

if ( ! -e "$infile" )
  { die "fm2mif: ERROR: Input file \"$infile\" does not exist... exiting"; }
if ( ! open( INFILE, "<$infile" ) )
  { die "fm2mif: ERROR: Could not open input file \"$infile\"... exiting"; }

$idline = <INFILE>;

if ( $idline =~ /^$FM_MAGIC/i || $idline =~ /^$FM_BOOK_MAGIC/i )
  { die "fm2mif: ERROR: File \"$infile\" is already in MIF format... exiting"; }
if ( $idline !~ /^$MIF_MAGIC/i && $idline !~ /^$MIF_BOOK_MAGIC/i )
  { die "fm2mif: ERROR: File \"$infile\" is not a FrameMaker document... exiting"; }

if ( $idline =~ /^$MIF_BOOK_MAGIC/i )
{
  local( @mifs, $oldfile );

  while ( <INFILE> )
  {
    if ( /<FileName[^`]*`<c\\>[^<]*<U\\>([^']*)'>/ )
    {
      local( $docpath, $mifpath );

      $docpath = $1;
      $mifpath = "$docpath.mif";

      if ( ! -e "$mifpath" )
      {
	local( $unstripped );
	$unstripped = $mifpath;
	$mifpath =~ s|\.doc||;

	if ( $mifpath eq $unstripped )
	{
	  print STDERR "mif2mif: ERROR: Input file \"$unstripped\" does not exist";
	  next;
	}
	elsif ( ! -e $mifpath )
	{
	  print STDERR "miftags: ERROR: Input file \"$unstripped\" does not exist, neither does \"$mifpath\"";
	  next;
	}
      }

      $mifs[++$#mifs] = $mifpath;
    }
  }

  @ARGV = @mifs;

  $mif = $infile;
  $doc = $mif;
  $doc =~ s/\.mif$//;
  $doc eq $mif && ($doc = "$mif.doc");

  if ( -e $doc )
  {
    local( $orig );

    $orig = ( -e "$doc.orig" ) ? "$doc.bak" : "$doc.orig"; 
    rename( "$doc", "$orig" ) &&
      (print STDERR "mif2mif: Existing file \"$doc\" renamed as \"$orig\"...");
  }

  print TMPF "Open $mif\nSaveAs d $mif $doc";
  print STDERR " $mif";
}

close( INFILE );

while ( $mif = shift )
{
  $doc = $mif;
  $doc =~ s/\.mif$//;
  $doc eq $mif && ($doc = "$mif.doc");

  if ( ! open( MIF, "<$mif" ) )
  {
    print STDERR "mif2fm: ERROR: Can't open file \"$mif\"";
    next;
  }

  $idline = <MIF>;
  close( MIF );

  if ( $idline =~ /^$FM_MAGIC/i || $idline =~ /^$FM_BOOK_MAGIC/i )
  {
    print STDERR "fm2mif: ATTENTION: File \"$mif\" is already in FrameMaker document format";
    next;
  }

  if ( $idline !~ /^$MIF_MAGIC/i && $idline !~ /^$MIF_BOOK_MAGIC/i )
  {
    print STDERR "fm2mif: ERROR: File \"$mif\" is not a FrameMaker file";
    next;
  }

  if ( -e $doc )
  {
    local( $orig );

    $orig = ( -e "$doc.orig" ) ? "$doc.bak" : "$doc.orig"; 
    rename( "$doc", "$orig" ) &&
      (print STDERR "mif2mif: Existing file \"$doc\" renamed as \"$orig\"...");
  }

  print TMPF "Open $mif\nSaveAs d $mif $doc";
  $inputs[++$#inputs] =  "  $mif";
}

print TMPF "Quit"; 
close( TMPF );

print STDERR 
  "fm2mif: Translating the following MIF files to FrameMaker documents:";
print @inputs;

system( "-FMBATCH- $tmpfile" );
system( "rm -f $tmpfile" );

print STDERR "mif2fm: Done";

